@@ -195,7 +195,7 @@ GUI :: GUI (Data& data, Queue& queue, ArticleCache& cache, Prefs& prefs, GroupPr
 
   //_group_pane = new GroupPane (*this, data, _prefs);
   _group_pane = new GroupPane (*this, data, _prefs);
-  _header_pane = new HeaderPane (*this, data, _queue, _cache, _prefs);
+  _header_pane = new HeaderPane (*this, data, _queue, _cache, _prefs, *this);
   _body_pane = new BodyPane (data, _cache, _prefs);
 
   std::string path = "/ui/main-window-toolbar";
@@ -751,7 +751,7 @@ void GUI :: do_download_selected_article ()
   const article_vector_t articles (_header_pane->get_full_selection_v ());
   Queue::tasks_t tasks;
   foreach_const (article_vector_t, articles, it)
-    tasks.push_back (new TaskArticle (_data, _data, **it, _cache, _data, this));
+    tasks.push_back (new TaskArticle (_data, _data, **it, _cache, _data));
   if (!tasks.empty())
     _queue.add_tasks (tasks, Queue::TOP);
 }
@@ -1145,8 +1145,6 @@ void GUI :: do_about_pan ()
   const gchar * authors [] = { "Charles Kerr", 0 };
   GdkPixbuf * logo = gdk_pixbuf_new_from_inline(-1, icon_pan_about_logo, 0, 0);
   GtkAboutDialog * w (GTK_ABOUT_DIALOG (gtk_about_dialog_new ()));
-  GdkPixbuf * icon = gdk_pixbuf_new_from_inline (-1, icon_pan, FALSE, 0);
-  gtk_window_set_icon (GTK_WINDOW(w), icon); 
   gtk_about_dialog_set_name (w, _("Pan"));
   gtk_about_dialog_set_version (w, PACKAGE_VERSION);
   gtk_about_dialog_set_comments (w, VERSION_TITLE);
@@ -1159,7 +1157,6 @@ void GUI :: do_about_pan ()
   g_signal_connect (G_OBJECT (w), "response", G_CALLBACK (gtk_widget_destroy), NULL);
   gtk_widget_show_all (GTK_WIDGET(w));
   g_object_unref (logo);
-  g_object_unref (icon);
 #else
   GtkWidget * dialog = gtk_dialog_new_with_buttons (PACKAGE_STRING,
                                                     GTK_WINDOW (get_window (_root)),
