@@ -67,6 +67,7 @@ const char * fallback_ui_file =
 "        <menuitem action='show-smilies-as-graphics' />\n"
 "        <menuitem action='show-text-markup'/>\n"
 "        <menuitem action='size-pictures-to-fit'/>\n"
+"        <menuitem action='focus-on-image'/>\n"
 "      </menu>\n"
 "      <separator />\n"
 "      <menu action='character-encoding-menu'>\n"
@@ -115,6 +116,7 @@ const char * fallback_ui_file =
 "        <separator />\n"
 "        <menuitem action='read-selected-article' />\n"
 "        <menuitem action='download-selected-article' />\n"
+"        <menuitem action='show-selected-article-info' />\n"
 "        <separator />\n"
 "        <menuitem action='mark-article-read' />\n"
 "        <menuitem action='mark-article-unread' />\n"
@@ -150,7 +152,7 @@ const char * fallback_ui_file =
 "    <toolitem action='get-new-headers-in-subscribed-groups'/>\n"
 "    <toolitem action='get-new-headers-in-selected-groups'/>\n"
 "    <separator />\n"
-"    <separator name='group-pane-filter'/>\n"
+"    <placeholder name='group-pane-filter'/>\n"
 "    <separator />\n"
 "    <toolitem action='post'/>\n"
 "    <toolitem action='followup-to'/>\n"
@@ -175,6 +177,7 @@ const char * fallback_ui_file =
 "    <separator />\n"
 "    <menuitem action='read-selected-article' />\n"
 "    <menuitem action='download-selected-article' />\n"
+"    <menuitem action='show-selected-article-info' />\n"
 "    <separator />\n"
 "    <menuitem action='mark-article-read' />\n"
 "    <menuitem action='mark-article-unread' />\n"
