@@ -92,7 +92,11 @@ namespace pan
       GtkWidget * _subject_entry;
       GtkWidget * _groups_entry;
       GtkWidget * _to_entry;
+      GtkWidget * _followupto_entry;
+      GtkWidget * _replyto_entry;
       GtkWidget * _body_view;
+      GtkWidget * _user_agent_check;
+      GtkWidget * _message_id_check;
       GtkTextBuffer * _body_buf;
       GtkTextBuffer * _headers_buf;
       GMimeMessage * _message;
@@ -108,6 +112,8 @@ namespace pan
       str2str_t _hidden_headers;
       str2str_t _profile_headers;
       std::string _unchanged_body;
+      int _wrap_pixels;
+      GtkTooltips * _ttips;
 
     private:
       void add_actions (GtkWidget* box);
@@ -121,10 +127,16 @@ namespace pan
       bool check_message (const Quark& server, GMimeMessage*);
       bool check_charset ();
 
+    private:
+      GtkWidget* create_main_tab ();
+      GtkWidget* create_extras_tab ();
+
     private:
       std::string get_body () const;
       gulong body_view_realized_handler;
-      static void body_view_realized (GtkWidget*, gpointer);
+      static void body_view_realized_cb (GtkWidget*, gpointer);
+      GtkWidget* create_body_widget (GtkTextBuffer*&, GtkWidget*&, const pan::Prefs&);
+      static void body_widget_resized_cb (GtkWidget*, GtkAllocation*, gpointer);
 
     private:
       unsigned long _group_entry_changed_id;
