@@ -27,9 +27,14 @@ extern "C" {
 #include "hig.h"
 #include "pad.h"
 #include "score-add-ui.h"
+#include "score-view-ui.h"
 
 using namespace pan;
 
+#if !GTK_CHECK_VERSION(2,6,0)
+#define GTK_STOCK_EDIT GTK_STOCK_OPEN
+#endif
+
 /**
 *** Key - Value paired comboboxes
 **/
@@ -156,13 +161,14 @@ namespace
   /**
   **/
 
-  enum Days { WEEK=7, MONTH=30, FOREVER=0 };
+  enum Days { WEEK=7, MONTH=31, MONTHS=93, FOREVER=0 };
 
   GtkTreeModel* time_tree_model_new ()
   {
     struct { int type; const char * str; } items[] = {
-      { WEEK,       N_("the next week") },
-      { MONTH,      N_("the next month") },
+      { WEEK,       N_("for the next week") },
+      { MONTH,      N_("for the next month") },
+      { MONTHS,     N_("for the next three months") },
       { FOREVER,    N_("forever") },
     };
 
@@ -448,12 +454,58 @@ ScoreAddDialog :: add_this_to_scorefile (bool do_rescore)
 void
 ScoreAddDialog :: response_cb (GtkDialog * w, int response, gpointer dialog_gpointer)
 {
+  bool add (false);
+  bool close (false);
   ScoreAddDialog * self (static_cast<ScoreAddDialog*>(dialog_gpointer));
 
-  if (response == GTK_RESPONSE_APPLY || response == GTK_RESPONSE_OK)
-    self->add_this_to_scorefile (response==GTK_RESPONSE_APPLY);
+  if (response == GTK_RESPONSE_CANCEL)
+  {
+    close = true;
+  }
+  else if ((response == GTK_RESPONSE_OK || response == GTK_RESPONSE_APPLY))
+  {
+    // are there any assignment items on this article already?
+    bool assign (false);
+    Scorefile::items_t items;
+    self->_data.get_article_scores (self->_group, self->_article, items);
+    for (Scorefile::items_t::const_iterator it(items.begin()), end(items.end()); !assign && it!=end; ++it)
+      assign |= it->value_assign_flag;
 
-  gtk_widget_destroy (GTK_WIDGET(w));
+    if (!assign) // no conflict...
+    {
+      add = close = true;
+    }
+    else
+    {
+      GtkWidget * d (gtk_message_dialog_new_with_markup (
+        GTK_WINDOW(self->_root),
+        GTK_DIALOG_DESTROY_WITH_PARENT,
+        GTK_MESSAGE_WARNING, GTK_BUTTONS_NONE, NULL));
+      HIG :: message_dialog_set_text (GTK_MESSAGE_DIALOG(d),
+        _("Another rule already sets this article's score."),
+        _("You may want to go back or delete the old rule."));
+      gtk_dialog_add_buttons (GTK_DIALOG(d),
+        GTK_STOCK_GO_BACK, -20,
+        GTK_STOCK_DELETE, -21,
+        GTK_STOCK_ADD, -22,
+        NULL);
+      const int conflict_response (gtk_dialog_run (GTK_DIALOG(d)));
+      gtk_widget_destroy (d);
+      if (conflict_response == -20) // go back
+        return;
+      else if (conflict_response == -22) // add
+        add = close = true;
+      else { // delete
+        ScoreView * score_view = new ScoreView (self->_data, GTK_WINDOW(w), self->_group, self->_article);
+        gtk_widget_show_all (score_view->root());
+      }
+    }
+  }
+
+  if (add)
+    self->add_this_to_scorefile (response==GTK_RESPONSE_APPLY);
+  if (close)
+    gtk_widget_destroy (GTK_WIDGET(w));
 }
 
 void
@@ -568,9 +620,13 @@ ScoreAddDialog :: ScoreAddDialog (Data           & data,
                                   Mode             mode):
   _data (data),
   _article (article),
+  _group (group),
   _root (0)
 {
-  GtkWidget * w = _root = gtk_dialog_new_with_buttons (_("Pan: New Scoring Rule"),
+  std::string s (_("Pan"));
+  s += ": ";
+  s += _("New Scoring Rule");
+  GtkWidget * w = _root = gtk_dialog_new_with_buttons (s.c_str(),
     GTK_WINDOW(gtk_widget_get_toplevel(parent)),
     GTK_DIALOG_DESTROY_WITH_PARENT,
     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
@@ -619,7 +675,7 @@ ScoreAddDialog :: ScoreAddDialog (Data           & data,
     GtkObject * a = gtk_adjustment_new (100, INT_MIN, INT_MAX, 1.0, 1.0, 1.0);
     w = _score_spin = gtk_spin_button_new (GTK_ADJUSTMENT(a), 100.0, 0u);
     gtk_box_pack_start (GTK_BOX(h), w, true, true, 0);
-    HIG::workarea_add_row (t, &row, _("Then"), h);
+    HIG::workarea_add_wide_control (t, &row, h);
     gtk_widget_show_all (h);
     g_signal_connect (_score_menu, "changed", G_CALLBACK(score_combo_changed_cb), w);
 
@@ -627,7 +683,7 @@ ScoreAddDialog :: ScoreAddDialog (Data           & data,
     model = time_tree_model_new ();
     w = _duration_menu = value_combo_new (model);
     g_object_unref (model);
-    HIG::workarea_add_row (t, &row, _("For"), w);
+    HIG::workarea_add_wide_control (t, &row, w);
     gtk_widget_show (w);
 
   populate (group, article, mode);
