@@ -79,7 +79,7 @@ void
 TaskGroups :: on_nntp_line (NNTP               * nntp,
                             const StringView   & line)
 {
-  char permission ('?');
+  char permission ('y');
   std::string name_str;
   std::string desc_str;
 
@@ -110,10 +110,12 @@ TaskGroups :: on_nntp_line (NNTP               * nntp,
   if (!name.empty()) {
     const Quark name_quark (name);
     Data::NewGroup& ng (_new_groups[name_quark]);
-    ng.group = name_quark;
-    ng.description.assign (desc.str, desc.len);
-    if (permission != '?')
-      ng.permission = permission;
+    if (ng.group.empty())
+        ng.group = name_quark;
+    if (ng.description.empty())
+        ng.description.assign (desc.str, desc.len);
+    if (ng.permission == '?')
+        ng.permission = permission;
   }
 
   if (!(++_group_count % 100ul)) {
@@ -126,14 +128,13 @@ TaskGroups :: on_nntp_line (NNTP               * nntp,
 }
 
 void
-TaskGroups :: on_nntp_done (NNTP    * nntp,
-                            Health    health)
+TaskGroups :: on_nntp_done (NNTP              * nntp,
+                            Health              health,
+                            const StringView  & response)
 {
   debug ("groups task got an on_nntp_done() from " << nntp->_server);
 
-  const bool is_ok (health == OK);
-
-  if (health == RETRY)
+  if (health == NETWORK_FAILED)
   {
     _state.set_need_nntp (_servername);
   }
@@ -159,5 +160,5 @@ TaskGroups :: on_nntp_done (NNTP    * nntp,
     }
   }
 
-  check_in (nntp, is_ok);
+  check_in (nntp, health);
 }
